@@ -22,6 +22,7 @@ to compute statistics such as its entropy.
 """
 
 import collections
+import warnings
 
 import chex
 import distrax
@@ -41,14 +42,21 @@ ContinuousDistribution = collections.namedtuple(
 
 def categorical_sample(key, probs):
   """Sample from a set of discrete probabilities."""
-  probs = probs / probs.sum(axis=-1, keepdims=True)
-  is_valid = jnp.logical_and(jnp.all(jnp.isfinite(probs)), jnp.all(probs >= 0))
-  sample = distrax.Categorical(probs=probs).sample(seed=key)
-  return jnp.where(is_valid, sample, -1)
+  warnings.warn(
+      "Rlax categorical_sample will be deprecated. "
+      "Please use distrax.Categorical.sample instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
+  return distrax.Categorical(probs=probs).sample(seed=key)
 
 
 def softmax(temperature=1.):
   """A softmax distribution."""
+  warnings.warn(
+      "Rlax softmax will be deprecated. "
+      "Please use distrax.Softmax instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
 
   def sample_fn(key: Array, logits: Array):
     return distrax.Softmax(logits, temperature).sample(seed=key)
@@ -72,6 +80,11 @@ def softmax(temperature=1.):
 def clipped_entropy_softmax(temperature=1., entropy_clip=1.):
   """A softmax distribution with clipped entropy (1 is eq to not clipping)."""
 
+  warnings.warn(
+      "Rlax clipped_entropy_softmax will be deprecated. "
+      "Please use distrax.Softmax instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
   def sample_fn(key: Array, logits: Array, action_spec=None):
     del action_spec
     return distrax.Softmax(logits, temperature).sample(seed=key)
@@ -105,7 +118,11 @@ def _mix_with_uniform(probs, epsilon):
 
 def epsilon_softmax(epsilon, temperature):
   """An epsilon-softmax distribution."""
-
+  warnings.warn(
+      "Rlax epsilon_softmax will be deprecated. "
+      "Please use distrax.Softmax instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
   def sample_fn(key: Array, logits: Array):
     probs = distrax.Softmax(logits=logits, temperature=temperature).probs
     return distrax.Categorical(
@@ -135,7 +152,11 @@ def epsilon_softmax(epsilon, temperature):
 
 def greedy():
   """A greedy distribution."""
-
+  warnings.warn(
+      "Rlax greedy will be deprecated. "
+      "Please use distrax.Greedy instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
   def sample_fn(key: Array, preferences: Array):
     return distrax.Greedy(preferences).sample(seed=key)
 
@@ -155,6 +176,11 @@ def greedy():
 def epsilon_greedy(epsilon=None):
   """An epsilon-greedy distribution."""
 
+  warnings.warn(
+      "Rlax epsilon_greedy will be deprecated. "
+      "Please use distrax.EpsilonGreedy instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
   def sample_fn(key: Array, preferences: Array, epsilon=epsilon):
     return distrax.EpsilonGreedy(preferences, epsilon).sample(seed=key)
 
@@ -172,6 +198,13 @@ def epsilon_greedy(epsilon=None):
 
 def safe_epsilon_softmax(epsilon, temperature):
   """Tolerantly handles the temperature=0 case."""
+
+  warnings.warn(
+      "Rlax safe_epsilon_softmax will be deprecated. "
+      "Please use distrax instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
+
   egreedy = epsilon_greedy(epsilon)
   unsafe = epsilon_softmax(epsilon, temperature)
 
@@ -202,14 +235,15 @@ def safe_epsilon_softmax(epsilon, temperature):
                               kl_fn)
 
 
-def _add_gaussian_noise(key, sample, sigma):
-  noise = jax.random.normal(key, shape=sample.shape) * sigma
-  return sample + noise
-
-
 def gaussian_diagonal(sigma=None):
   """A gaussian distribution with diagonal covariance matrix."""
 
+  warnings.warn(
+      "Rlax gaussian_diagonal will be deprecated. "
+      "Please use distrax MultivariateNormalDiag instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
+
   def sample_fn(key: Array, mu: Array, sigma: Array = sigma):
     return distrax.MultivariateNormalDiag(
         mu, jnp.ones_like(mu) * sigma).sample(seed=key)
@@ -244,13 +278,12 @@ def gaussian_diagonal(sigma=None):
 def squashed_gaussian(sigma_min=-4, sigma_max=0.):
   """A squashed gaussian distribution with diagonal covariance matrix."""
 
-  def minmaxvals(a, action_spec):
-    # broadcasts action spec to action shape
-    min_shape = action_spec.minimum.shape or (1,)
-    max_shape = action_spec.maximum.shape or (1,)
-    min_vals = jnp.broadcast_to(action_spec.minimum, a.shape[:-1] + min_shape)
-    max_vals = jnp.broadcast_to(action_spec.maximum, a.shape[:-1] + max_shape)
-    return min_vals, max_vals
+  warnings.warn(
+      "Rlax squashed_gaussian will be deprecated. "
+      "Please use distrax Transformed MultivariateNormalDiag distribution "
+      "with chained Tanh/ScalarAffine bijector instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
 
   def sigma_activation(sigma, sigma_min=sigma_min, sigma_max=sigma_max):
     return jnp.exp(sigma_min + 0.5 * (sigma_max - sigma_min) *
@@ -259,87 +292,40 @@ def squashed_gaussian(sigma_min=-4, sigma_max=0.):
   def mu_activation(mu):
     return jnp.tanh(mu)
 
-  def transform(a, action_spec):
-    min_vals, max_vals = minmaxvals(a, action_spec)
-    scale = (max_vals - min_vals) * 0.5
-    actions = (jnp.tanh(a) + 1.0) * scale + min_vals
-    return actions
+  def get_squashed_gaussian_dist(mu, sigma, action_spec=None):
+    if action_spec is not None:
+      scale = 0.5 * (action_spec.maximum - action_spec.minimum)
+      shift = action_spec.minimum
+      bijector = distrax.Chain([distrax.ScalarAffine(shift=shift, scale=scale),
+                                distrax.ScalarAffine(shift=1.0),
+                                distrax.Tanh()])
+    else:
+      bijector = distrax.Tanh()
+    return distrax.Transformed(
+        distribution=distrax.MultivariateNormalDiag(
+            loc=mu_activation(mu), scale_diag=sigma_activation(sigma)),
+        bijector=distrax.Block(bijector, ndims=1))
 
-  def inv_transform(a, action_spec):
-    min_vals, max_vals = minmaxvals(a, action_spec)
-    scale = (max_vals - min_vals) * 0.5
-    actions_tanh = (a - min_vals) / scale - 1.
-    return jnp.arctanh(actions_tanh)
-
-  def log_det_jacobian(a, action_spec):
-    min_vals, max_vals = minmaxvals(a, action_spec)
-    scale = (max_vals - min_vals) * 0.5
-    log_ = jnp.sum(jnp.log(scale))
-    # computes sum log (1-tanh(a)**2)
-    log_ += jnp.sum(2. * (jnp.log(2.) - a - jax.nn.softplus(-2. * a)))
-    return log_
-
-  def sample_fn(key: Array,
-                mu: Array,
-                sigma: Array,
-                action_spec,
-                ):
-    mu = mu_activation(mu)
-    sigma = sigma_activation(sigma)
-    action = _add_gaussian_noise(key, mu, sigma)
-    return transform(action, action_spec)
+  def sample_fn(key: Array, mu: Array, sigma: Array, action_spec):
+    return get_squashed_gaussian_dist(mu, sigma, action_spec).sample(seed=key)
 
   def prob_fn(sample: Array, mu: Array, sigma: Array, action_spec):
-    # Support scalar and vector `sigma`. If vector, mu.shape==sigma.shape.
-    mu = mu_activation(mu)
-    sigma = sigma_activation(sigma)
-    # Compute pdf for multivariate gaussian.
-    d = mu.shape[-1]
-    det = jnp.prod(sigma**2, axis=-1)
-    z = ((2 * jnp.pi)**(0.5 * d)) * (det**0.5)
-    exp = jnp.exp(-0.5 * jnp.sum(
-        ((mu - inv_transform(sample, action_spec)) / sigma)**2, axis=-1))
-    det_jacobian = jnp.prod(jnp.clip(1 - sample**2, 0., 1.) + 1e-6)
-    return exp / (z * det_jacobian)
+    return get_squashed_gaussian_dist(mu, sigma, action_spec).prob(sample)
 
   def logprob_fn(sample: Array, mu: Array, sigma: Array, action_spec):
-    # Support scalar and vector `sigma`. If vector, mu.shape==sigma.shape.
-    mu = mu_activation(mu)
-    sigma = sigma_activation(sigma)
-    # Compute logpdf for multivariate gaussian in a numerically safe way.
-    d = mu.shape[-1]
-    half_logdet = jnp.sum(jnp.log(sigma), axis=-1)
-    logz = half_logdet + 0.5 * d * jnp.log(2 * jnp.pi)
-    logexp = -0.5 * jnp.sum(
-        ((mu - inv_transform(sample, action_spec)) / sigma)**2, axis=-1)
-    return logexp - logz - log_det_jacobian(sample, action_spec)
+    return get_squashed_gaussian_dist(mu, sigma, action_spec).log_prob(sample)
 
   def entropy_fn(mu: Array, sigma: Array):
-    # Support scalar and vector `sigma`. If vector, mu.shape==sigma.shape.
-    mu = mu_activation(mu)
-    sigma = sigma_activation(sigma)
-    # Compute entropy in a numerically safe way.
-    d = mu.shape[-1]
-    half_logdet = jnp.sum(jnp.log(sigma), axis=-1)
-    return half_logdet + 0.5 * d * (1 + jnp.log(2 * jnp.pi))
+    return get_squashed_gaussian_dist(mu, sigma).distribution.entropy()
 
-  def kl_to_standard_normal_fn(mu: Array,
-                               sigma: Array,
-                               per_dimension: bool = False):
-    mu = mu_activation(mu)
-    sigma = sigma_activation(sigma)
-    v = jnp.clip(sigma**2, 1e-6, 1e6)
-    kl = 0.5 * (v + mu**2 - jnp.ones_like(mu) - jnp.log(v))
-    if not per_dimension:
-      kl = jnp.sum(kl, axis=-1)
-    return kl
+  def kl_to_standard_normal_fn(mu: Array, sigma: Array):
+    return get_squashed_gaussian_dist(mu, sigma).distribution.kl_divergence(
+        distrax.MultivariateNormalDiag(
+            jnp.zeros_like(mu), jnp.ones_like(mu)))
 
   def kl_fn(mu_0: Array, sigma_0: Numeric, mu_1: Array, sigma_1: Numeric):
-    sigma_0 = sigma_activation(sigma_0)
-    mu_0 = mu_activation(mu_0)
-    sigma_1 = sigma_activation(sigma_1)
-    mu_1 = mu_activation(mu_1)
-    return multivariate_normal_kl_divergence(mu_0, sigma_0, mu_1, sigma_1)
+    return get_squashed_gaussian_dist(mu_0, sigma_0).distribution.kl_divergence(
+        get_squashed_gaussian_dist(mu_1, sigma_1).distribution)
 
   return ContinuousDistribution(sample_fn, prob_fn, logprob_fn, entropy_fn,
                                 kl_to_standard_normal_fn, kl_fn)
@@ -358,6 +344,11 @@ def categorical_importance_sampling_ratios(pi_logits_t: Array,
   Returns:
     importance sampling ratios.
   """
+  warnings.warn(
+      "Rlax categorical_importance_sampling_ratios will be deprecated. "
+      "Please use distrax.importance_sampling_ratios instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
   return distrax.importance_sampling_ratios(distrax.Categorical(
       pi_logits_t), distrax.Categorical(mu_logits_t), a_t)
 
@@ -369,7 +360,10 @@ def categorical_cross_entropy(
   """Computes the softmax cross entropy between sets of logits and labels.
 
   See "Deep Learning" by Goodfellow et al.
-  (http://www.deeplearningbook.org/contents/prob.html).
+  (http://www.deeplearningbook.org/contents/prob.html). The computation is
+  equivalent to:
+
+                  sum_i (labels_i * log_softmax(logits_i))
 
   Args:
     labels: a valid probability distribution (non-negative, sum to 1).
@@ -378,8 +372,13 @@ def categorical_cross_entropy(
   Returns:
     a scalar loss.
   """
-  chex.assert_rank([logits, labels], 1)
-  return -jnp.sum(labels * jax.nn.log_softmax(logits))
+  warnings.warn(
+      "Rlax categorical_cross_entropy will be deprecated. "
+      "Please use distrax.Categorical.cross_entropy instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
+  return distrax.Categorical(probs=labels).cross_entropy(
+      distrax.Categorical(logits=logits))
 
 
 def categorical_kl_divergence(
@@ -397,6 +396,11 @@ def categorical_kl_divergence(
   Returns:
     the kl divergence between the distributions.
   """
+  warnings.warn(
+      "Rlax categorical_kl_divergence will be deprecated. "
+      "Please use distrax.Softmax.kl_divergence instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
   return distrax.Softmax(p_logits, temperature).kl_divergence(
       distrax.Softmax(q_logits, temperature))
 
@@ -415,5 +419,10 @@ def multivariate_normal_kl_divergence(
   Returns:
     the kl divergence between the distributions.
   """
+  warnings.warn(
+      "Rlax multivariate_normal_kl_divergence will be deprecated."
+      "Please use distrax.MultivariateNormalDiag.kl_divergence instead.",
+      PendingDeprecationWarning, stacklevel=2
+  )
   return distrax.MultivariateNormalDiag(mu_0, sigma_0).kl_divergence(
       distrax.MultivariateNormalDiag(mu_1, sigma_1))
