@@ -16,9 +16,11 @@
 
 import functools
 import math
+from typing import Tuple
 
 from absl.testing import absltest
 from absl.testing import parameterized
+import chex
 import haiku as hk
 import jax
 import jax.numpy as jnp
@@ -27,6 +29,9 @@ import optax
 from rlax._src import distributions
 from rlax._src import mpo_ops
 
+Array = chex.Array
+Numeric = chex.Numeric
+
 NUM_SAMPLES = 10
 ACTION_DIM = 3
 TIME_DIM = 8
@@ -75,7 +80,7 @@ def _hk_mock_policy_params(s_tm1):
 
 
 def _init_params(key):
-  init_fn, _ = hk.transform(_hk_mock_policy_params, apply_rng=True)
+  init_fn, _ = hk.transform(_hk_mock_policy_params)
   key_seq = hk.PRNGSequence(key)
   s_tm1 = jax.random.normal(
       next(key_seq), (TIME_DIM, BATCH_DIM, ACTION_DIM), jnp.float32)
@@ -92,7 +97,7 @@ def _init_params(key):
 
 def _mock_outputs(online_params, target_params, key, target_name):
   """Returns mock network outputs."""
-  _, policy_params_fn = hk.transform(_hk_mock_policy_params, apply_rng=True)
+  _, policy_params_fn = hk.transform(_hk_mock_policy_params)
   key_seq = hk.PRNGSequence(key)
 
   state_size = ACTION_DIM
@@ -152,10 +157,44 @@ def get_common_loss_fn_inputs(params, key, target_name):
           params['mpo']['temperature'], _EPSILON_BOUND)}
 
 
+def _decoupled_multivariate_normal_kl_divergence(
+    mu_0: Array, sigma_0: Numeric, mu_1: Array, sigma_1: Numeric,
+    per_dimension: bool = False
+) -> Tuple[Array, Array]:
+  """Compute the KL between diagonal Gaussians decomposed into mean and covar-e.
+
+  Args:
+    mu_0: array like of mean values for policy 0
+    sigma_0: array like of std values for policy 0
+    mu_1: array like of mean values for policy 1
+    sigma_1: array like of std values for policy 1
+    per_dimension: Whether to return a separate kl divergence for each dimension
+      on the last axis.
+
+  Returns:
+    the kl divergence between the distributions decomposed into mean and
+    covariance.
+  """
+  # Support scalar and vector `sigma`. If vector, mu.shape==sigma.shape.
+  sigma_1 = jnp.ones_like(mu_1) * sigma_1
+  sigma_0 = jnp.ones_like(mu_0) * sigma_0
+  v1 = jnp.clip(sigma_1**2, 1e-6, 1e6)
+  v0 = jnp.clip(sigma_0**2, 1e-6, 1e6)
+  mu_diff = mu_1 - mu_0
+  kl_mean = 0.5 * jnp.divide(mu_diff**2, v1)
+  kl_cov = 0.5 * (jnp.divide(v0, v1) - jnp.ones_like(mu_1) + jnp.log(v1) -
+                  jnp.log(v0))
+  if not per_dimension:
+    kl_mean = jnp.sum(kl_mean, axis=-1)
+    kl_cov = jnp.sum(kl_cov, axis=-1)
+
+  return kl_mean, kl_cov
+
+
 def get_decoupled_kl_constraints(out, params, per_dimension):
   """Factorises KL for Gaussian."""
   kl_mean, kl_covariance = (
-      distributions.decoupled_multivariate_normal_kl_divergence(
+      _decoupled_multivariate_normal_kl_divergence(
           out['target_pi_params']['mean'], out['target_pi_params']['stddev'],
           out['pi_params']['mean'], out['pi_params']['stddev'],
           per_dimension=per_dimension))
@@ -175,7 +214,7 @@ def get_decoupled_kl_constraints(out, params, per_dimension):
 
 def get_coupled_kl_constraints(out, params, per_dimension):
   kl_mean, kl_covariance = (
-      distributions.decoupled_multivariate_normal_kl_divergence(
+      _decoupled_multivariate_normal_kl_divergence(
           out['target_pi_params']['mean'], out['target_pi_params']['stddev'],
           out['pi_params']['mean'], out['pi_params']['stddev'],
           per_dimension=per_dimension))
