@@ -553,7 +553,7 @@ class CategoricalTDLearningTest(parameterized.TestCase):
     # Test outputs.
     actual = categorical_td_learning(
         self.logits_tm1, self.r_t, self.discount_t, self.logits_t)
-    np.testing.assert_allclose(self.expected, actual, rtol=1e-5)
+    np.testing.assert_allclose(self.expected, actual, rtol=1e-4)
 
 
 class CategoricalQLearningTest(parameterized.TestCase):
@@ -609,7 +609,7 @@ class CategoricalQLearningTest(parameterized.TestCase):
     categorical_q_learning = self.variant(jax.vmap(fn))
     # Test outputs.
     actual = categorical_q_learning(*self.inputs)
-    np.testing.assert_allclose(self.expected, actual, rtol=1e-5)
+    np.testing.assert_allclose(self.expected, actual, rtol=1e-4)
 
 
 class CategoricalDoubleQLearningTest(parameterized.TestCase):
@@ -669,7 +669,7 @@ class CategoricalDoubleQLearningTest(parameterized.TestCase):
     categorical_double_q_learning = self.variant(jax.vmap(fn))
     # Test outputs.
     actual = categorical_double_q_learning(*self.inputs)
-    np.testing.assert_allclose(self.expected, actual, rtol=1e-5)
+    np.testing.assert_allclose(self.expected, actual, rtol=1e-4)
 
   @chex.all_variants()
   def test_single_double_q_learning_eq_batch(self):
@@ -748,7 +748,7 @@ class QuantileRegressionLossTest(parameterized.TestCase):
       ('huber', 2.))
   def test_quantile_regression_loss_batch(self, huber_param):
     """Tests for a full batch."""
-    loss_fn = value_learning._quantile_regression_loss
+    loss_fn = value_learning.quantile_regression_loss
     loss_fn = self.variant(jax.vmap(functools.partial(
         loss_fn, huber_param=huber_param)))
     # Compute quantile regression loss.
@@ -827,29 +827,30 @@ class QuantileLearningTest(parameterized.TestCase):
     self.expected = {}
     for huber_param in [0.0, 1.0]:
       self.expected[huber_param] = np.array(  # loop over batch
-          [value_learning._quantile_regression_loss(dqa, tau, dt, huber_param)
+          [value_learning.quantile_regression_loss(dqa, tau, dt, huber_param)
            for (dqa, tau, dt) in zip(dist_qa_tm1, self.tau_q_tm1, dist_target)],
           dtype=np.float32)
 
     # Scenario 2:
-    # bootstrap targets are nor an argmax, but averaging across actions
+    # bootstrap targets are not an argmax, but averaging across actions
     self.uniform_probs_a_t = np.ones((3, 4), dtype=np.float32) / 4.
-    #                                             [ 2.25, 0.25]
-    # dist_qa_t                              =    [-0.75, 0.25]
-    #                                             [-3.00, 1.00]
-    uniform_dist_target = np.array(
-        [[1.625, 0.625],
-         [-1, -1],
-         [-3, 1]],
+
+    # all_action_targets = r + gamma * dist_q_t  (batch x n_taus x n_actions)
+    dts = np.array(
+        [[[0.5, 3, 1.5, 1.5], [0.5, -1, 1.5, 1.5]],
+         [[-1, -1, -1, -1], [-1, -1, -1, -1]],
+         [[-2, 2, -5, -7], [1, 3, 2, -2]]],
         dtype=np.float32)
 
     self.uniform_expected = {}
     for huber_param in [0.0, 1.0]:
-      self.uniform_expected[huber_param] = np.array(  # loop over batch
-          [value_learning._quantile_regression_loss(dqa, tau, dt, huber_param)
-           for (dqa, tau, dt) in zip(dist_qa_tm1, self.tau_q_tm1,
-                                     uniform_dist_target)],
-          dtype=np.float32)
+      all_action_losses = [  # loop over actions and batch
+          [value_learning.quantile_regression_loss(dqa, tau, dt, huber_param)
+           for (dqa, tau, dt) in zip(dist_qa_tm1, self.tau_q_tm1, dts[:, :, a])]
+          for a in range(4)]
+      # uniform probabilities means the loss is averaged over actions
+      self.uniform_expected[huber_param] = np.array(
+          all_action_losses, dtype=np.float32).mean(axis=0)
 
   @chex.all_variants()
   @parameterized.named_parameters(
